﻿module CsvProcessor

open System

let getCsvDataFromFile (filename : string) = 
  seq {
    use sr = new System.IO.StreamReader(filename, System.Text.Encoding.GetEncoding("shift_jis"))
    sr.ReadLine() |> ignore
    while not sr.EndOfStream do
      let oneLine = sr.ReadLine()
      yield oneLine.Split(',') |> Array.map (fun (v : string) -> v.Trim(' ').Trim('"'))
  }

let writeCsvDataToFile (outFile : string) (s : string array seq) =
  use outStream = new System.IO.StreamWriter(outFile, false, System.Text.Encoding.GetEncoding("shift_jis"))
  Seq.map (fun record -> Array.map string record |> Array.reduce (fun s v -> s + "," + v)) s
    |> Seq.iter (fun s -> outStream.WriteLine(s))

let null2Zero value _ = 
 if value = "" then "0" else value

let adder i1 i2 _ (ar : string []) =
 let val1 = Int32.TryParse(ar.[i1])
 let val2 = Int32.TryParse(ar.[i2])
 match val1, val2 with
 | (false, _), _ -> failwithf "数値に変換できない値があります(%s)" (ar.[i1])
 | _, (false, _) -> failwithf "数値に変換できない値があります(%s)" (ar.[i2])
 | (_, v1), (_,v2) -> string (v1 + v2)

let checker (f : string -> string[] -> string option) value (ar : string []) =
 match f value ar with
 | Some err -> failwith err
 | None -> value



let columnOperation value ar checks = 
  List.fold (fun stat check -> check stat ar) value checks

let csvOperation inFile outFile checkers =
  getCsvDataFromFile inFile
    |> Seq.map (fun record -> 
          List.mapi (fun idx colchks -> let value = columnOperation record.[idx] record colchks
                                        record.[idx] <- value
                                        value
                    ) checkers |> List.toArray) 
    |> writeCsvDataToFile outFile
